package net.sf.twip;

import org.junit.runners.model.*;

/**
 * Allows to wrap a JUnit- {@link Statement} {@link #wrapInner(FrameworkMethod, Object, Statement)
 * before} or {@link #wrapOuter(FrameworkMethod, Object, Statement) after} the execution of other
 * setup code.
 * 
 * @see TwipExtensions
 */
public abstract class TwipExtension {

	/**
	 * Decorate a JUnit {@link Statement} into a new one and thereby extend the functionality of
	 * JUnit. Your Statement is executed directly around the method call itself.
	 * 
	 * @param method
	 *            The test method that is to be called.
	 * @param testInstance
	 *            The test instance on which the method is called.
	 * @param statement
	 *            The {@link Statement} that you can wrap.
	 * @return a new {@link Statement} wrapping the one passed in; or directly that, if you choose
	 *         not to wrap anything.
	 */
	public Statement wrapInner(FrameworkMethod method, Object testInstance, Statement statement) {
		return statement;
	}

	/**
	 * Decorate a JUnit {@link Statement} into a new one and thereby extend the functionality of
	 * JUnit. Your Statement is executed around all the other setups/teardowns.
	 * 
	 * @param method
	 *            The test method that is to be called.
	 * @param testInstance
	 *            The test instance on which the method is called.
	 * @param statement
	 *            The {@link Statement} that you can wrap.
	 * @return a new {@link Statement} wrapping the one passed in; or directly that, if you choose
	 *         not to wrap anything.
	 */
	public Statement wrapOuter(FrameworkMethod method, Object testInstance, Statement statement) {
		return statement;
	}
}
