package net.sf.twip.mockito;

import net.sf.twip.TwipExtension;

import org.junit.runners.model.*;
import org.mockito.Mock;

/**
 * Allows you to use the {@link org.mockito.Mockito mockito} {@link Mock @Mock} annotation.
 */
public class MockitoExtension extends TwipExtension {

	private boolean checked = false;
	private boolean found = false;

	private synchronized boolean mockitoPresent() {
		if (!checked) {
			checked = true;
			try {
				Class.forName("org.mockito.Mockito");
				found = true;
			} catch (final ClassNotFoundException e) {
				found = false;
			}
		}
		return found;
	}

	public Statement wrapInner(FrameworkMethod method, Object testInstance, Statement statement) {
		if (!mockitoPresent())
			return statement;
		return new MockitoValidateStatement(statement);
	}

	public Statement wrapOuter(FrameworkMethod method, Object testInstance, Statement statement) {
		if (!mockitoPresent())
			return statement;
		return new MockitoInitStatement(statement, testInstance);
	}
}
